#include "lcd_2w.h"

#define F_CPU 16000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <stddef.h>
#include <stdlib.h>
#include "i2cmaster.h"

#define  LCDADDR  0x70

#define PCF_RS  0x01
#define PCF_RW  0x02
#define PCF_EN  0x04
#define PCF_BACKLIGHT 0x08

#define RSMODE_CMD  0
#define RSMODE_DATA 1

#define true	1
#define false	0

#define LOW		0
#define HIGH	1

char	_backlight;
uint8_t _displayfunction; ///< lines and dots mode
uint8_t _displaycontrol;  ///< cursor, display, blink flags
uint8_t _displaymode;     ///< left2right, autoscroll

uint8_t _numlines;        ///< The number of rows the display supports.
uint8_t _numcols;

void LCD_write(uint8_t halfByte, uint8_t mode, uint8_t enable) {
	// map the given values to the hardware of the I2C schema
	uint8_t i2cData = halfByte << 4;
	if (mode > 0) i2cData |= PCF_RS;
	// PCF_RW is never used.
	if (enable > 0) i2cData |= PCF_EN;
	if (_backlight > 0) i2cData |= PCF_BACKLIGHT;

	i2c_start(LCDADDR + I2C_WRITE);
	i2c_write(i2cData);
	i2c_stop();
}

void LCDsendNibble(uint8_t halfByte, uint8_t mode) {
	LCD_write(halfByte, mode, true);
	_delay_us(1);    // enable pulse must be >450ns
	LCD_write(halfByte, mode, false);
	_delay_us(37);   // commands need > 37us to settle
}

void LCDsend(uint8_t value, uint8_t mode) {
	// separate the 4 value-nibbles
	uint8_t valueLo = value    & 0x0F;
	uint8_t valueHi = value>>4 & 0x0F;

	LCDsendNibble(valueHi, mode);
	LCDsendNibble(valueLo, mode);
} 

inline void command(uint8_t value) {
	LCDsend(value, RSMODE_CMD);
} 

void LCDputChar(uint8_t value) {
	LCDsend(value, RSMODE_DATA);
}
void LCDsendString(char *string) {
	uint8_t i = 0;
	while (string[i]) {
		LCDputChar(string[i]);
		i++;
	}
}

void LCD_SendHex(int32_t data) {
	char ldata[16] = "";
	itoa(data, (char*) ldata, 16);
	LCDsendString((char *)ldata);
}


void LCDcreateChar(uint8_t location, uint8_t charmap[]) {
	location &= 0x7; // we only have 8 locations 0-7
	command(LCD_SETCGRAMADDR | (location << 3));
	for (int i=0; i<8; i++) {
		LCDputChar(charmap[i]);
	}
}

void LCDsetBacklight(uint8_t brightness) {
	_backlight = brightness;
	// send no data but set the background-pin right;
	LCD_write(0x00, RSMODE_DATA, false);
}

void LCDnoAutoscroll(void) {
	_displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
	command(LCD_ENTRYMODESET | _displaymode);
}
void LCDautoscroll(void) {
	_displaymode |= LCD_ENTRYSHIFTINCREMENT;
	command(LCD_ENTRYMODESET | _displaymode);
}

void LCDrightToLeft(void) {
	_displaymode &= ~LCD_ENTRYLEFT;
	command(LCD_ENTRYMODESET | _displaymode);
}

void LCDleftToRight(void) {
	_displaymode |= LCD_ENTRYLEFT;
	command(LCD_ENTRYMODESET | _displaymode);
}

void LCDscrollDisplayRight(void) {
	command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

void LCDscrollDisplayLeft(void) {
	command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}

void blink() {
	_displaycontrol |= LCD_BLINKON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void noBlink() {
	_displaycontrol &= ~LCD_BLINKON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void cursor() {
	_displaycontrol |= LCD_CURSORON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void noCursor() {
	_displaycontrol &= ~LCD_CURSORON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void display() {
	_displaycontrol |= LCD_DISPLAYON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void noDisplay() {
	_displaycontrol &= ~LCD_DISPLAYON;
	command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void LCDsetCursor(uint8_t col, uint8_t row)
{
	int row_offsets[] = { 0x00, 0x40, 0x14, 0x54   };
	if ( row >= _numlines ) {
		row = _numlines-1;    // we count rows starting w/0
	}

	command(LCD_SETDDRAMADDR | (col + row_offsets[row]));
}

void home(){
	command(LCD_RETURNHOME);  // set cursor position to zero
	_delay_us(2000);  // this command takes a long time!
}

void LCDclear(){
	command(LCD_CLEARDISPLAY);  // clear display, set cursor position to zero
	_delay_us(2000);  // this command takes a long time!
}

void LCD_init(uint8_t cols, uint8_t lines, uint8_t dotsize) {
	// cols ignored !
	_numlines = lines;
	_numcols = cols;
	_displayfunction = 0;

	if (lines > 1) {
		_displayfunction |= LCD_2LINE;
	}

	// for some 1 line displays you can select a 10 pixel high font
	if ((dotsize != 0) && (lines == 1)) {
		_displayfunction |= LCD_5x10DOTS;
	}

	// SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!
	// according to datasheet, we need at least 40ms after power rises above 2.7V
	// before sending commands. Arduino can turn on way befor 4.5V so we'll wait 50
	i2c_init();

	// initializing th display
	LCD_write(0x00, LOW, false);
	_delay_us(50000);

	// put the LCD into 4 bit mode according to the hitachi HD44780 datasheet figure 26, pg 47
	LCDsendNibble(0x03, RSMODE_CMD);
	_delay_us(4500);
	LCDsendNibble(0x03, RSMODE_CMD);
	_delay_us(4500);
	LCDsendNibble(0x03, RSMODE_CMD);
	_delay_us(150);
	// finally, set to 4-bit interface
	LCDsendNibble(0x02, RSMODE_CMD);

	// finally, set # lines, font size, etc.
	command(LCD_FUNCTIONSET | _displayfunction);

	// turn the display on with no cursor or blinking default
	_displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
	display();

	// clear it off
	LCDclear();

	// Initialize to default text direction (for romance languages)
	_displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;
	// set the entry mode
	command(LCD_ENTRYMODESET | _displaymode);
}




